@@ -47,15 +47,6 @@
 #include "gtkstatusbox.h"
 #include "pidginstock.h"
 
-#ifdef USE_HILDON
-#	define SCROLLABLE_CX_MAX -1
-#	if HILDON_CHECK_VERSION(2,1,0)
-#		define SCROLLABLE_CY_MAX 320
-#	else /* HILDON < 2.1.0 */
-#		define SCROLLABLE_CY_MAX 220
-#	endif /* HILDON >= 2.1.0 */
-#endif /* USE_HILDON */
-
 enum
 {
 	COLUMN_ICON,
@@ -827,7 +818,7 @@ add_protocol_options(AccountPrefsDialog *dialog)
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), PIDGIN_HIG_BORDER);
 #ifdef USE_HILDON
 	gtk_notebook_insert_page(GTK_NOTEBOOK(dialog->notebook),
-			_pidgin_make_container_scrollable(vbox, SCROLLABLE_CX_MAX, SCROLLABLE_CY_MAX),
+			_pidgin_make_container_scrollable(vbox),
 			gtk_label_new_with_mnemonic(_("Ad_vanced")), 1);
 #else /* !USE_HILDON */
 	gtk_notebook_insert_page(GTK_NOTEBOOK(dialog->notebook), vbox,
@@ -1565,7 +1556,11 @@ pidgin_account_dialog_show(PidginAccountDialogType type,
 	main_vbox = pidgin_dialog_get_vbox_with_properties(GTK_DIALOG(win), FALSE, PIDGIN_HIG_BOX_SPACE);
 
 	dialog->notebook = notebook = gtk_notebook_new();
+#ifdef USE_HILDON
+	gtk_box_pack_start(GTK_BOX(main_vbox), notebook, TRUE, TRUE, 0);
+#else /* !USE_HILDON */
 	gtk_box_pack_start(GTK_BOX(main_vbox), notebook, FALSE, FALSE, 0);
+#endif /* USE_HILDON */
 	gtk_widget_show(GTK_WIDGET(notebook));
 
 	/* Setup the inner vbox */
@@ -1573,7 +1568,7 @@ pidgin_account_dialog_show(PidginAccountDialogType type,
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), PIDGIN_HIG_BORDER);
 #ifdef USE_HILDON
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			_pidgin_make_container_scrollable(vbox, SCROLLABLE_CX_MAX, SCROLLABLE_CY_MAX),
+			_pidgin_make_container_scrollable(vbox),
 			gtk_label_new_with_mnemonic(_("_Basic")));
 #else /* !USE_HILDON */
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox,
@@ -1604,7 +1599,7 @@ pidgin_account_dialog_show(PidginAccountDialogType type,
 	gtk_container_set_border_width(GTK_CONTAINER(dbox), PIDGIN_HIG_BORDER);
 #ifdef USE_HILDON
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			_pidgin_make_container_scrollable(dbox, SCROLLABLE_CX_MAX, SCROLLABLE_CY_MAX),
+			_pidgin_make_container_scrollable(dbox),
 			gtk_label_new_with_mnemonic(_("P_roxy")));
 #else /* !USE_HILDON */
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), dbox,
