@@ -24,6 +24,9 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
  */
 #include "internal.h"
+#ifdef USE_HILDON
+#	define LEAVE_TOGGLE_BUTTONS_ALONE
+#endif /* USE_HILDON */
 #include "pidgin.h"
 
 #include "notify.h"
@@ -36,7 +39,6 @@
 #include "gtkimhtml.h"
 #include "gtkutils.h"
 #include "pidginstock.h"
-#include "gtkblist.h"
 
 #ifdef HAVE_REGEX_H
 # include <regex.h>
@@ -95,7 +97,7 @@ debug_window_destroy(GtkWidget *w, GdkEvent *event, void *unused)
 	if(debug_win->timer != 0) {
 		const gchar *text;
 
-		g_source_remove(debug_win->timer);
+		purple_timeout_remove(debug_win->timer);
 
 		text = gtk_entry_get_text(GTK_ENTRY(debug_win->expression));
 		purple_prefs_set_string(PIDGIN_PREFS_ROOT "/debug/regex", text);
@@ -232,10 +234,10 @@ save_writefile_cb(void *user_data, const char *filename)
 static void
 save_cb(GtkWidget *w, DebugWindow *win)
 {
-	purple_request_file_with_hint(win, _("Save Debug Log"), "purple-debug.log", TRUE,
+	purple_request_file(win, _("Save Debug Log"), "purple-debug.log", TRUE,
 					  G_CALLBACK(save_writefile_cb), NULL,
 					  NULL, NULL, NULL,
-					  PURPLE_REQUEST_UI_HINT_BLIST, win);
+					  win);
 }
 
 static void
@@ -553,7 +555,7 @@ regex_changed_cb(GtkWidget *w, DebugWindow *win) {
 	}
 
 	if(win->timer == 0)
-		win->timer = purple_timeout_add(5000, (GSourceFunc)regex_timer_cb, win);
+		win->timer = purple_timeout_add_seconds(5, (GSourceFunc)regex_timer_cb, win);
 
 	regex_compile(win);
 }
@@ -665,6 +667,9 @@ toolbar_context(GtkWidget *toolbar, GdkEventButton *event, gpointer null)
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	}
 
+#ifdef USE_HILDON
+	_hildon_make_menu_thumbable(menu);
+#endif /* USE_HILDON */
 	gtk_widget_show_all(menu);
 
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
@@ -695,7 +700,6 @@ toolbar_tap_and_hold_cb (GtkWidget *tb, gpointer data)
 static DebugWindow *
 debug_window_new(void)
 {
-	PidginBuddyList *blist;
 	DebugWindow *win;
 	GtkWidget *vbox;
 	GtkWidget *toolbar;
@@ -709,16 +713,15 @@ debug_window_new(void)
 	width  = purple_prefs_get_int(PIDGIN_PREFS_ROOT "/debug/width");
 	height = purple_prefs_get_int(PIDGIN_PREFS_ROOT "/debug/height");
 
-	PIDGIN_DIALOG(win->window);
-	if ((blist = pidgin_blist_get_default_gtk_blist()) != NULL)
-		if (blist->window)
-			gtk_window_set_transient_for(GTK_WINDOW(win->window), GTK_WINDOW(blist->window));
+#ifdef USE_HILDON
+	win->window = pidgin_create_window(_("Debug Window"), 0, "debug", TRUE);
+#else /* !USE_HILDON */
+	win->window = pidgin_create_dialog(_("Debug Window"), 0, "debug", TRUE);
+#endif /* USE_HILDON */
 	purple_debug_info("gtkdebug", "Setting dimensions to %d, %d\n",
 					width, height);
 
 	gtk_window_set_default_size(GTK_WINDOW(win->window), width, height);
-	gtk_window_set_role(GTK_WINDOW(win->window), "debug");
-	gtk_window_set_title(GTK_WINDOW(win->window), _("Debug Window"));
 
 	g_signal_connect(G_OBJECT(win->window), "delete_event",
 	                 G_CALLBACK(debug_window_destroy), NULL);
@@ -726,7 +729,7 @@ debug_window_new(void)
 	                 G_CALLBACK(configure_cb), win);
 
 	handle = pidgin_debug_get_handle();
-	
+
 #ifdef HAVE_REGEX_H
 	/* the list store for all the messages */
 	win->store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_INT);
@@ -742,8 +745,12 @@ debug_window_new(void)
 #endif /* HAVE_REGEX_H */
 
 	/* Setup the vbox */
-	vbox = gtk_vbox_new(FALSE, 0);
+#ifdef USE_HILDON
+	vbox = g_object_new(GTK_TYPE_VBOX, "visible", TRUE, NULL);
 	gtk_container_add(GTK_CONTAINER(win->window), vbox);
+#else /* !USE_HILDON */
+	vbox = pidgin_dialog_get_vbox(GTK_DIALOG(win->window));
+#endif /* USE_HILDON */
 
 	if (purple_prefs_get_bool(PIDGIN_PREFS_ROOT "/debug/toolbar")) {
 		/* Setup our top button bar thingie. */
@@ -803,7 +810,7 @@ debug_window_new(void)
 		win->filter =
 			gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
 									   GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
-									   NULL, _("Filter"), _("Filter"), 
+									   NULL, _("Filter"), _("Filter"),
 									   NULL, NULL,
 									   G_CALLBACK(regex_filter_toggled_cb),
 									   win);
@@ -855,7 +862,7 @@ debug_window_new(void)
 		                           GTK_TOOLBAR_CHILD_WIDGET, gtk_label_new(_("Level ")),
 		                           NULL, _("Select the debug filter level."),
 		                           NULL, NULL, NULL, NULL);
-		
+
 		win->filterlevel = gtk_combo_box_new_text();
 		gtk_toolbar_append_element(GTK_TOOLBAR(toolbar),
 		                           GTK_TOOLBAR_CHILD_WIDGET, win->filterlevel,
@@ -1022,6 +1029,9 @@ pidgin_debug_init(void)
 	REGISTER_G_LOG_HANDLER("GModule");
 	REGISTER_G_LOG_HANDLER("GLib-GObject");
 	REGISTER_G_LOG_HANDLER("GThread");
+#ifdef USE_GSTREAMER
+	REGISTER_G_LOG_HANDLER("GStreamer");
+#endif
 
 #ifdef _WIN32
 	if (!purple_debug_is_enabled())
