@@ -548,7 +548,7 @@ pidgin_pounce_editor_show(PurpleAccount *account, const char *name,
 	/* Create the vbox that will contain all the prefs stuff. */
 	vbox2 = gtk_vbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
 #ifdef USE_HILDON
-	gtk_box_pack_start(GTK_BOX(vbox1), _pidgin_make_container_scrollable(vbox2, -1, -1), TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox1), _pidgin_make_container_scrollable(vbox2), TRUE, TRUE, 0);
 #else /* !USE_HILDON */
 	gtk_box_pack_start(GTK_BOX(vbox1), vbox2, TRUE, TRUE, 0);
 #endif /* USE_HILDON */
