@@ -1201,7 +1201,7 @@ pidgin_status_editor_show(gboolean edit, PurpleSavedStatus *saved_status)
 	vbox1 = g_object_new(GTK_TYPE_VBOX, "visible", TRUE, "spacing", PIDGIN_HIG_BORDER, NULL);
 	g_object_set(G_OBJECT(vbox), "spacing", 0, NULL);
 	gtk_container_add(GTK_CONTAINER(vbox),
-		_pidgin_make_container_scrollable(vbox1, -1, -1));
+		_pidgin_make_container_scrollable(vbox1));
 	vbox = vbox1;
 #endif /* USE_HILDON */
 
@@ -1269,7 +1269,7 @@ pidgin_status_editor_show(gboolean edit, PurpleSavedStatus *saved_status)
 #endif /* !USE_HILDON */
 	gtk_box_pack_start(GTK_BOX(dbox), 
 #ifdef USE_HILDON
-		_pidgin_make_container_scrollable(pidgin_make_scrollable(dialog->treeview, GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS, GTK_SHADOW_IN, -1, -1), -1, -1),
+		_pidgin_make_container_scrollable(pidgin_make_scrollable(dialog->treeview, GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS, GTK_SHADOW_IN, -1, -1)),
 #else /* !USE_HILDON */
 		pidgin_make_scrollable(dialog->treeview, GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS, GTK_SHADOW_IN, -1, -1),
 #endif /* USE_HILDON */
