@@ -40,6 +40,8 @@
 #include "notify.h"
 #include "prefs.h"
 #include "sound.h"
+#include "sound-theme.h"
+#include "theme-manager.h"
 #include "util.h"
 
 #include "gtkconv.h"
@@ -59,9 +61,11 @@ static gboolean gst_init_failed;
 #endif /* USE_GSTREAMER */
 
 #ifdef USE_HILDON
+#if !HILDON_CHECK_VERSION(1,0,0)
 /* Cannot play multiple sounds at once, because it causes hangage :o( */
 static gboolean sound_in_progress = FALSE ;
-#endif /* def USE_HILDON */
+#endif /* HILDON < 1.0.0 */
+#endif /* USE_HILDON */
 
 static const struct pidgin_sound_event sounds[PURPLE_NUM_SOUNDS] = {
 	{N_("Buddy logs in"), "login", "login.wav"},
@@ -126,7 +130,7 @@ play_conv_event(PurpleConversation *conv, PurpleSoundEventID event)
 		gtkconv = PIDGIN_CONVERSATION(conv);
 		has_focus = purple_conversation_has_focus(conv);
 
-		if (!(gtkconv && gtkconv->make_sound) ||
+		if (!gtkconv->make_sound ||
 			(has_focus && !purple_prefs_get_bool(PIDGIN_PREFS_ROOT "/sound/conv_focus")))
 		{
 			return;
@@ -161,7 +165,7 @@ im_msg_sent_cb(PurpleAccount *account, const char *receiver,
 			   const char *message, PurpleSoundEventID event)
 {
 	PurpleConversation *conv = purple_find_conversation_with_account(
-		PURPLE_CONV_TYPE_ANY, receiver, account);
+		PURPLE_CONV_TYPE_IM, receiver, account);
 	play_conv_event(conv, event);
 }
 
@@ -229,9 +233,9 @@ static void
 account_signon_cb(PurpleConnection *gc, gpointer data)
 {
 	if (mute_login_sounds_timeout != 0)
-		g_source_remove(mute_login_sounds_timeout);
+		purple_timeout_remove(mute_login_sounds_timeout);
 	mute_login_sounds = TRUE;
-	mute_login_sounds_timeout = purple_timeout_add(10000, unmute_login_sounds_cb, NULL);
+	mute_login_sounds_timeout = purple_timeout_add_seconds(10, unmute_login_sounds_cb, NULL);
 }
 
 const char *
@@ -299,6 +303,7 @@ pidgin_sound_init(void)
 	purple_prefs_add_path(PIDGIN_PREFS_ROOT "/sound/file/nick_said", "");
 	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/enabled/pounce_default", TRUE);
 	purple_prefs_add_path(PIDGIN_PREFS_ROOT "/sound/file/pounce_default", "");
+	purple_prefs_add_string(PIDGIN_PREFS_ROOT "/sound/theme", "");
 	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/conv_focus", TRUE);
 	purple_prefs_add_bool(PIDGIN_PREFS_ROOT "/sound/mute", FALSE);
 	purple_prefs_add_path(PIDGIN_PREFS_ROOT "/sound/command", "");
@@ -377,8 +382,10 @@ bus_call (GstBus     *bus,
 		gst_element_set_state(play, GST_STATE_NULL);
 		gst_object_unref(GST_OBJECT(play));
 #ifdef USE_HILDON
+#if !HILDON_CHECK_VERSION(1,0,0)
 		sound_in_progress = FALSE ;
-#endif /* def USE_HILDON */
+#endif /* HILDON < 1.0.0 */
+#endif /* USE_HILDON */
 		break;
 	case GST_MESSAGE_WARNING:
 		gst_message_parse_warning(msg, &err, NULL);
@@ -412,6 +419,12 @@ expire_old_child(gpointer data)
 }
 #endif
 
+#define PLAY_SOUND_NORMALLY
+#ifdef USE_HILDON
+#	if !HILDON_CHECK_VERSION(1,0,0)
+#		undef PLAY_SOUND_NORMALLY
+#	endif /* HILDON < 1.0.0 */
+#endif /* USE_HILDON */
 static void
 pidgin_sound_play_file(const char *filename)
 {
@@ -419,11 +432,11 @@ pidgin_sound_play_file(const char *filename)
 #ifdef USE_GSTREAMER
 	float volume;
 	char *uri;
-#ifdef USE_HILDON /* Should this be USE_MAEMO ? */
-	GError *error = NULL ;
-#else /* !USE_HILDON */
+#ifdef PLAY_SOUND_NORMALLY
 	GstElement *sink = NULL;
-#endif /* USE_HILDON */
+#else /* !PLAY_SOUND_NORMALLY */
+	GError *error = NULL ;
+#endif /* PLAY_SOUND_NORMALLY */
 	GstElement *play = NULL;
 	GstBus *bus = NULL;
 #endif
@@ -459,7 +472,7 @@ pidgin_sound_play_file(const char *filename)
 		if (!sound_cmd || *sound_cmd == '\0') {
 			purple_debug_error("gtksound",
 					 "'Command' sound method has been chosen, "
-					 "but no command has been set.");
+					 "but no command has been set.\n");
 			return;
 		}
 
@@ -498,7 +511,8 @@ pidgin_sound_play_file(const char *filename)
 #ifdef USE_GSTREAMER
 	if (gst_init_failed)  /* Perhaps do gdk_beep instead? */
 		return;
-#ifdef USE_HILDON /* Should this be USE_MAEMO ? */
+#ifdef USE_HILDON
+#if !HILDON_CHECK_VERSION(1,0,0)
 	if (sound_in_progress)
 		return ;
 	volume = (((float)((CLAMP(purple_prefs_get_int(PIDGIN_PREFS_ROOT "/sound/volume"),0,100))) * 65535.0) / 100.0);
@@ -520,32 +534,20 @@ pidgin_sound_play_file(const char *filename)
 	}
 
 	sound_in_progress = TRUE ;
-#else /* !USE_HILDON */
+#else /* HILDON > 1.0.0 */
+	volume = (float)(CLAMP(purple_prefs_get_int(PIDGIN_PREFS_ROOT "/sound/volume"),0,100)) / 25;
+#endif /* HILDON < 1.0.0 */
+#endif /* USE_HILDON */
+#ifdef PLAY_SOUND_NORMALLY
 	volume = (float)(CLAMP(purple_prefs_get_int(PIDGIN_PREFS_ROOT "/sound/volume"),0,100)) / 50;
 	if (!strcmp(method, "automatic")) {
-		if (purple_running_gnome()) {
-			sink = gst_element_factory_make("gconfaudiosink", "sink");
-		}
-		if (!sink)
-			sink = gst_element_factory_make("autoaudiosink", "sink");
-		if (!sink) {
-			purple_debug_error("sound", "Unable to create GStreamer audiosink.\n");
-			return;
-		}
+		sink = gst_element_factory_make("gconfaudiosink", "sink");
 	}
 #ifndef _WIN32
 	else if (!strcmp(method, "esd")) {
 		sink = gst_element_factory_make("esdsink", "sink");
-		if (!sink) {
-			purple_debug_error("sound", "Unable to create GStreamer audiosink.\n");
-			return;
-		}
 	} else if (!strcmp(method, "alsa")) {
 		sink = gst_element_factory_make("alsasink", "sink");
-		if (!sink) {
-			purple_debug_error("sound", "Unable to create GStreamer audiosink.\n");
-			return;
-		}
 	}
 #endif
 	else {
@@ -553,6 +555,11 @@ pidgin_sound_play_file(const char *filename)
 		return;
 	}
 
+	if (strcmp(method, "automatic") != 0 && !sink) {
+		purple_debug_error("sound", "Unable to create GStreamer audiosink.\n");
+		return;
+	}
+
 	play = gst_element_factory_make("playbin", "play");
 
 	if (play == NULL) {
@@ -564,7 +571,7 @@ pidgin_sound_play_file(const char *filename)
 	g_object_set(G_OBJECT(play), "uri", uri,
 		                     "volume", volume,
 		                     "audio-sink", sink, NULL);
-#endif /* USE_HILDON */
+#endif /* PLAY_SOUND_NORMALLY */
 
 	bus = gst_pipeline_get_bus(GST_PIPELINE(play));
 	gst_bus_add_watch(bus, bus_call, play);
@@ -604,6 +611,8 @@ pidgin_sound_play_event(PurpleSoundEventID event)
 {
 	char *enable_pref;
 	char *file_pref;
+	const char *theme_name;
+	PurpleSoundTheme *theme;
 
 	if ((event == PURPLE_SOUND_BUDDY_ARRIVE) && mute_login_sounds)
 		return;
@@ -620,13 +629,31 @@ pidgin_sound_play_event(PurpleSoundEventID event)
 	/* check NULL for sounds that don't have an option, ie buddy pounce */
 	if (purple_prefs_get_bool(enable_pref)) {
 		char *filename = g_strdup(purple_prefs_get_path(file_pref));
-		if(!filename || !strlen(filename)) {
+		theme_name = purple_prefs_get_string(PIDGIN_PREFS_ROOT "/sound/theme");
+
+		if (theme_name && *theme_name && (!filename || !*filename)) {
+			/* Use theme */
 			g_free(filename);
+
+			theme = PURPLE_SOUND_THEME(purple_theme_manager_find_theme(theme_name, "sound"));
+			filename = purple_sound_theme_get_file_full(theme, sounds[event].pref);
+
+			if(!g_file_test(filename, G_FILE_TEST_IS_REGULAR)){ /* Use Default sound in this case */
+				purple_debug_error("sound", "The file: (%s) %s\n from theme: %s, was not found or wasn't readable\n",
+							sounds[event].pref, filename, theme_name);
+				g_free(filename);
+			}
+		}
+
+		if (!filename || !strlen(filename)) {			    /* Use Default sounds */
+			g_free(filename);
+
 			/* XXX Consider creating a constant for "sounds/purple" to be shared with Finch */
 			filename = g_build_filename(DATADIR, "sounds", "purple", sounds[event].def, NULL);
 		}
 
 		purple_sound_play_file(filename, NULL);
+
 		g_free(filename);
 	}
 
@@ -634,6 +661,29 @@ pidgin_sound_play_event(PurpleSoundEventID event)
 	g_free(file_pref);
 }
 
+gboolean
+pidgin_sound_is_customized(void)
+{
+	gint i;
+	gchar *path, *file;
+
+	for (i = 0; i < PURPLE_NUM_SOUNDS; i++) {
+		path = g_strdup_printf(PIDGIN_PREFS_ROOT "/sound/file/%s", sounds[i].pref);
+		file = g_strdup(purple_prefs_get_path(path));
+		g_free(path);
+
+		if (file && file[0] != '\0'){
+			g_free(file);
+			return TRUE;
+		}
+
+		g_free(file);
+	}
+
+	return FALSE;
+
+}
+
 static PurpleSoundUiOps sound_ui_ops =
 {
 	pidgin_sound_init,
