@@ -1751,6 +1751,7 @@ treeview_cursor_changed_cb(GtkTreeView *treeview, gpointer data)
 	gtk_tree_path_free (cursor);
 }
 
+#ifdef USE_HILDON
 #if HILDON_CHECK_VERSION(2,1,0)
 static void
 treeview_row_activated_cb(GtkTreeView *tv, GtkTreePath *tp, GtkTreeViewColumn *col, PidginStatusBox *status_box)
@@ -1758,6 +1759,7 @@ treeview_row_activated_cb(GtkTreeView *tv, GtkTreePath *tp, GtkTreeViewColumn *c
 	treeview_activate_current_selection(status_box, tp);
 }
 #endif /* HILDON >= 2.1.0 */
+#endif /* USE_HILDON */
 
 static void
 pidgin_status_box_init (PidginStatusBox *status_box)
@@ -1828,9 +1830,11 @@ pidgin_status_box_init (PidginStatusBox *status_box)
 	gtk_widget_show (status_box->popup_frame);
 
 	status_box->tree_view = gtk_tree_view_new ();
+#ifdef USE_HILDON
 #if HILDON_CHECK_VERSION(2,1,0)
 	g_signal_connect(G_OBJECT(status_box->tree_view), "row-activated", (GCallback)treeview_row_activated_cb, status_box);
 #endif /* HILDON >= 2.1.0 */
+#endif /* USE_HILDON */
 	sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (status_box->tree_view));
 	gtk_tree_selection_set_mode (sel, GTK_SELECTION_BROWSE);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (status_box->tree_view),
